<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Action;

use Psr\Log\InvalidArgumentException;
use Rebilly\Wp\Member\Exception\AjaxFailException;
use Rebilly\Wp\Member\Exception\AjaxResponseException;
use Rebilly\Wp\Member\Exception\AjaxSuccessException;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

abstract class AjaxAction
{
    /** @var string Action name */
    protected $name;

    /** @var bool Is this action available from frontend */
    protected $public;

    /** @var bool Is nonce check required */
    protected $useNonce;

    /**
     * @param array $data
     *
     * @throws AjaxResponseException
     * @return array
     */
    abstract public function process(array $data): array;

    public function handle(): void
    {
        $data = $_REQUEST;

        if (!$this->verifyNonce($data)) {
            wp_send_json([
                'result' => false,
                'message' => __('Session has expired', REBILLY_WP_LANG_DOMAIN),
            ]);
        }

        $statusCode = null;

        try {
            $response = $this->process($data);
        } catch (AjaxResponseException $ex) {
            $response = [
                'success' => $ex->isSuccess(),
            ];
            if ($ex->isHasMessage()) {
                $response['message'] = $ex->getMessage();
            }
            if ($ex->getErrors() !== null) {
                $response['errors'] = $ex->getErrors();
            }
            $response = array_merge($response, $ex->getAdditional());
            $statusCode = $ex->getCode() ?: null;
        }

        wp_send_json($response, $statusCode);
    }

    /**
     * @param string $suffix
     *
     * @return string
     */
    public function generateNonce($suffix = ''): ?string
    {
        return $this->useNonce ? wp_create_nonce($this->name . $suffix) : null;
    }

    /**
     * @return string
     */
    public function getName(): string
    {
        return $this->name;
    }

    /**
     * AjaxAction constructor.
     * @param string $name
     * @param bool $public
     * @param bool $useNonce
     */
    protected function __construct(string $name, bool $public = false, bool $useNonce = true)
    {
        $this->name = $name;
        $this->public = $public;
        $this->useNonce = $useNonce;

        $this->register();
    }

    protected function register(): void
    {
        $handler = [$this, 'handle'];
        add_action("wp_ajax_{$this->name}", $handler);

        if ($this->public) {
            add_action("wp_ajax_nopriv_{$this->name}", $handler);
        }
    }

    /**
     * @param array $data
     *
     * @return bool
     */
    protected function verifyNonce(array $data): bool
    {
        return !$this->useNonce || wp_verify_nonce($data['_nonce'], $this->name);
    }

    /**
     * @param array $filtersRaw
     * @return string
     */
    protected function buildFilters(array $filtersRaw): string
    {
        $filters = [];
        foreach ($filtersRaw as $key => $value) {
            $filters[] = "{$key}:{$value}";
        }

        return implode(';', $filters);
    }

    /**
     * @param string|array|null $message
     * @param array $additional
     *
     * @throws AjaxSuccessException
     */
    protected function success($message, array $additional = []): void
    {
        $message = $this->compileMessage($message);

        throw new AjaxSuccessException($message, $additional);
    }

    /**
     * @param string|array|null $message
     * @param array|null $errors
     *
     * @throws AjaxFailException
     */
    protected function fail($message, ?array $errors = null): void
    {
        $message = $this->compileMessage($message);

        throw new AjaxFailException($message, $errors);
    }

    /**
     * @param string|array|null $message
     * @return string
     */
    protected function compileMessage($message): ?string
    {
        if ($message === null) {
            return null;
        }

        $messageRaw = is_array($message) ? $message[0] ?? null : $message;
        if ($message === null) {
            throw new InvalidArgumentException('Message can\'t be null');
        }
        $messageLocale = __($messageRaw, REBILLY_WP_LANG_DOMAIN);
        $compiled = is_array($message) ?
            sprintf($messageLocale, ...array_slice($message, 1)) :
            $messageLocale;

        return $compiled;
    }
}
