<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Action;

use DomainException;
use PasswordHash;
use Rebilly\Entities\Customer;
use Rebilly\Entities\PaymentCardToken;
use Rebilly\Entities\Plan;
use Rebilly\Entities\Subscription as ApiSubscription;
use Rebilly\Entities\Subscriptions\PlanItem;
use Rebilly\Entities\Transaction;
use Rebilly\Http\Exception\UnprocessableEntityException;
use Rebilly\Rest\Collection;
use Rebilly\Wp\Member\Entity\Form;
use Rebilly\Wp\Member\Entity\Subscription;
use Rebilly\Wp\Member\Entity\User;
use Rebilly\Wp\Member\Exception\AjaxFailException;
use Rebilly\Wp\Member\Plugin;
use Rebilly\Wp\Member\Settings;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class Subscribe
 * @package Rebilly\Wp\Member\Action
 */
class Subscribe extends AjaxAction
{
    public function __construct()
    {
        parent::__construct('rebilly_form_subscribe', true);
    }

    public function process(array $data): array
    {
        $form = Form::find($data['_form']);
        if (!$form) {
            $this->fail('Form not found');
        }

        $user = User::find();

        if (!$user && $form->isAuthenticationRequired()) {
            $this->fail('You must be authorized to subscribe');
        }

        $validation = $form->validate($data);
        if ($validation->fails()) {
            $this->fail('Provided data is invalid', $validation->errors()->firstOfAll());
        }

        try {
            $subscription = $this->subscribe($user, $form, $data);

            $isActivated = $subscription->getStatus() === 'active';

            $additional = [];
            $redirectPage = $form->getPageUrl($isActivated ? 'success' : 'pending');
            if ($redirectPage) {
                $additional['redirect'] = [
                    'url' => $redirectPage,
                ];
            }
            $this->success(
                $isActivated ?
                    'Your subscription activated' :
                    'Your subscription will be activated soon',
                $additional
            );
        } catch (UnprocessableEntityException $e) {
            $this->fail('Unexpected error, try again', ['form' => $e->getErrors()]);
        }

        return [
            'success' => false,
            'message' => __('Unexpected error, try again', REBILLY_WP_LANG_DOMAIN),
        ];
    }

    /**
     * @param User|null $user
     * @param string|null $email
     * @param PaymentCardToken $token
     * @param array $form
     *
     * @return Customer
     * @throws AjaxFailException
     * @throws UnprocessableEntityException
     */
    private function maybeCreateCustomer(?User $user, ?string $email, PaymentCardToken $token, array $form): Customer
    {
        /** @var bool|Customer $customer */
        $customer = false;
        $customerId = false;
        $customerExisted = true;
        if ($user &&
            ($customerId = $user->getCustomerId()) &&
            !($customer = Plugin::api()->customer()->get($customerId))) {
            // Nothing we should do. Just create new customer, but clear this field before
            // so we will not try this again
            $user->setCustomerId(false);
        }
        if (!$customer) {
            try {
                // Search customer by his email
                if ($email) {
                    $customers = Plugin::api()->customer()->findByEmail($email);
                    if ($customers->count()) {
                        $customer = $customers->offsetGet(0);
                    }
                }

                if (!$customer) {
                    // Create new one if he is not found by email address
                    $customer = Plugin::api()
                        ->customer()
                        ->create(
                            (new Customer())
                                ->setPaymentToken($token->getId())
                                ->setPrimaryAddress($form['billingAddress'] ?? [])
                                ->setCustomFields($form['customFields']['customer'] ?? [])
                        );
                    $customerExisted = false;
                }
            } catch (UnprocessableEntityException $e) {
                // Can only be validation error, so just print it out
                $this->fail('Please fill in the form', ['form' => $e->getErrors()]);
            }
        }

        if ($customerExisted) {
            $payload = [
                'paymentToken' => $token->getId(),
            ];
            if (!$customer->getPrimaryAddress()) {
                $payload['primaryAddress'] = $form;
            }
            $customer = Plugin::api()
                ->customer()
                ->update($customer->getId(), $payload);
        }

        // We created/changed customerId and need to save it
        if ($user && $customerId !== $customer->getId()) {
            $user->setCustomerId($customer->getId());
        }

        return $customer;
    }

    /**
     * @param string $token
     *
     * @return PaymentCardToken
     * @throws AjaxFailException
     */
    private function getToken(string $token): PaymentCardToken
    {
        $rebilly = Plugin::api();
        /** @noinspection CallableParameterUseCaseInTypeContextInspection */
        $token = $rebilly->paymentCard()->getToken($token);
        if (!$token || $token->getIsUsed()) {
            // Second time we try to create PaymentCard from token
            $this->fail('Payment card data expired, please try again');
        }

        return $token;
    }

    /**
     * @param Collection|Plan[] $plansRaw
     *
     * @return array
     */
    private function getSubscriptionItems(array $plansRaw): array
    {
        $plans = [];
        foreach ($plansRaw as $planId => $quantity) {
            $plans[] = [
                'planId' => $planId,
                'quantity' => $quantity,
            ];
        }

        return $plans;
    }

    /**
     * @param array $plansRaw
     *
     * @return array|string[]
     */
    private function getProducts(array $plansRaw): array
    {
        $ids = [];
        $plans = Plugin::api()
            ->plan()
            ->all(array_keys($plansRaw));
        foreach ($plans as $plan) {
            $ids[] = $plan->getProductId();
        }

        return $ids;
    }

    /**
     * @param array $data
     * @param Customer $customer
     * @param bool $userExisted
     *
     * @return User
     * @throws AjaxFailException
     */
    private function maybeCreateUser(array $data, Customer $customer, bool &$userExisted = false): User
    {
        $user = User::findByCustomerId($customer->getId()) ?: User::findByEmail($data['email']);
        $userExisted = true;

        if (!$user) {
            $userExisted = false;
            $user = User::create($data);
            if (is_wp_error($user)) {
                $this->fail($user->get_error_message());
            }
        }

        $user->setCustomerId($customer->getId());

        return $user;
    }

    private function getPlans(Form $form, array $rawData): array
    {
        $bumpFields = $form->getBumpFields();
        $addonFields = $form->getAddonFields();

        $isBump = false;
        foreach ($bumpFields as $name => $field) {
            if (!blank($rawData[$name] ?? '')) {
                $isBump = true;

                break;
            }
        }

        $plans = [];
        $quantities = $form->getPlanQuantityFields();
        foreach ($form->getPlanIds($isBump) as $plan) {
            $plans[$plan['plan']] = $this->getPlanQuantity($plan, $quantities, $rawData);
        }

        $addonPlans = [];
        foreach ($addonFields as $name => $field) {
            if (!blank($rawData[$name] ?? '')) {
                $addonPlans[] = $form->getAddonPlanIds($field['settings']['addon']);
            }
        }
        $plans = array_merge($plans, ...$addonPlans);

        return $plans;
    }

    private function getPlanQuantity(array $plan, array $quantities, array $rawData): int
    {
        if (!$plan['mode']) {
            return (int) $plan['quantity'];
        }

        $field = $quantities[$plan['plan']] ?? '';
        if (isset($rawData[$field]) && is_numeric($rawData[$field]) && !blank($field)) {
            return (int) min(100, max(1, $rawData[$field]));
        }

        return 1;
    }

    /**
     * @param User|null $user
     * @param Form $form
     * @param array $rawData
     *
     * @return ApiSubscription
     * @throws AjaxFailException
     * @throws UnprocessableEntityException
     */
    private function subscribe(?User $user, Form $form, array $rawData): ApiSubscription
    {
        $formFields = $form->getRebillyFields($rawData, $email);
        $framePayToken = $rawData['rebilly-token'];

        $token = $this->getToken($framePayToken);
        $plans = $this->getPlans($form, $rawData);
        $subscriptionItems = $this->getSubscriptionItems($plans);
        $products = $this->getProducts($plans);

        $customer = $this->maybeCreateCustomer($user, $email, $token, $formFields);

        $customFields = $formFields['customFields']['subscription'] ?? [];

        $customFieldNames = Plugin::rulesEngine()->getFields();

        $userExisted = $user !== null;
        $userNeedsActivation = true;

        try {
            $websiteId = Settings::get('website_id');
            $website = Plugin::api()->website()->getFromAll($websiteId);

            if (!$userExisted && $form->isSignupEnabled()) {
                $user = $this->maybeCreateUser($rawData, $customer, $userExisted);
                if (!$user->user_activation_key) {
                    $userNeedsActivation = false;
                }

                if ($userNeedsActivation && !empty($customFieldNames) && $form->isRebillyConfirmationStrategy()) {
                    Plugin::api()
                        ->customer()
                        ->update($customer->getId(), [
                            'customFields' => [
                                    $customFieldNames['username'] => $user->user_nicename,
                                ] + $customer->getCustomFields(),
                        ]);

                    $key = wp_generate_password(20, false);

                    do_action('retrieve_password_key', $user->user_login, $key);

                    global $wpdb, $wp_hasher;
                    if (empty($wp_hasher)) {
                        require_once ABSPATH . WPINC . '/class-phpass.php';
                        $wp_hasher = new PasswordHash(8, true);
                    }
                    $hashed = time() . ':' . $wp_hasher->HashPassword($key);
                    $wpdb->update($wpdb->users, ['user_activation_key' => $hashed], ['user_login' => $user->user_login]);

                    $customFields[$customFieldNames['email-confirmation']] = true;
                    $customFields[$customFieldNames['email-confirmation-url']] = network_site_url("wp-login.php?action=rp&key=$key&login=" . rawurlencode($user->user_login), 'login');
                }
            }

            $customFields[$customFieldNames['is-wp']] = true;

            $subscription = $this->maybeCreateSubscription($customer, $websiteId, $formFields, $subscriptionItems, $customFields);

            $isApproved = false;

            try {
                if (!$subscription) {
                    throw new DomainException('Subscription is not created');
                }
                $invoice = Plugin::api()->invoice()->get($subscription->getInitialInvoiceId());
                if (!$invoice) {
                    throw new DomainException('Initial invoice not found');
                }
                $transaction = Plugin::api()
                    ->transaction()
                    ->create(
                        (new Transaction())
                            ->setWebsiteId($websiteId)
                            ->setCustomerId($customer->getId())
                            ->setCurrency($invoice->getCurrency())
                            ->setType(Transaction::TYPE_SALE)
                            ->setAmount($invoice->getAmount())
                            ->setInvoiceIds([$invoice->getId()])
                    );
                if (!$transaction) {
                    throw new DomainException('Transaction is not created');
                }
                if ($transaction->getResult() === 'declined') {
                    if (!$userExisted && $user) {
                        wp_delete_user($user->ID);
                    }

                    if ($website && $website->getServicePhone() !== '') {
                        $this->fail('Your payment has been declined, please confirm your details and try again, or call support at ' . $website->getServicePhone());
                    } elseif ($website->getServiceEmail()) {
                        $this->fail('Your payment has been declined, please confirm your details and try again, or contact support at ' . $website->getServiceEmail());
                    } else {
                        $this->fail('Your payment has been declined, please confirm your details and try again');
                    }
                }

                if (isset($formFields['autoPay']) && !((bool) $formFields['autoPay'])) {
                    $subscription->setAutopay(false);
                    $subscription = Plugin::api()
                        ->subscription()
                        ->update($subscription->getId(), $subscription);
                }

                $isApproved = $transaction->getResult() === 'approved';

                $subscription = Plugin::api()
                    ->subscription()
                    ->get($subscription->getId());
            } catch (DomainException $ex) {
                if (!$userExisted && $user) {
                    wp_delete_user($user->ID);
                }
                $this->fail('Unexpected error, try again');
            }

            if ($userNeedsActivation && (empty($customFieldNames) || !$form->isRebillyConfirmationStrategy())) {
                wp_new_user_notification($user, null, 'user');
            }

            if ($user && $isApproved) {
                $user->grantCapability($products);
            }
        } catch (UnprocessableEntityException $ex) {
            if (!$userExisted && $user) {
                wp_delete_user($user->ID);
            }

            throw $ex;
        }
        // Save subscription to site database for future processing
        Subscription::create($user, $subscription, $plans, $products, $form, $formFields);

        return $subscription;
    }

    private function maybeCreateSubscription(Customer $customer, $websiteId, array $formFields, array $subscriptionItems, array $customFields): ApiSubscription
    {
        $subscriptions = Plugin::api()
            ->subscription()
            ->findPendingByCustomer($customer->getId());

        foreach ($subscriptions as $subscription) {
            if ($this->subscriptionItemsSame($subscription->getItems(), $subscriptionItems)) {
                return $subscription;
            }
        }

        $billingAddress = $formFields['billingAddress'] ?? [];
        $deliveryAddress = $formFields['deliveryAddress'] ?? $billingAddress;

        $subscription = Plugin::api()
            ->subscription()
            ->create((new ApiSubscription())
                ->setCustomerId($customer->getId())
                ->setWebsiteId($websiteId)
                ->setItems($subscriptionItems)
                ->setBillingAddress($billingAddress)
                ->setDeliveryAddress($deliveryAddress)
                ->setCustomFields($customFields)
                ->setAutopay(true));

        return $subscription;
    }

    /**
     * @param array|PlanItem[] $actual
     * @param array $expected
     * @return bool
     */
    private function subscriptionItemsSame(array $actual, array $expected): bool
    {
        $map = [];
        foreach ($expected as $item) {
            $map[$item['planId']] = (int) $item['quantity'];
        }
        foreach ($actual as $plan) {
            if (isset($map[$plan->getPlanId()]) && $map[$plan->getPlanId()] === $plan->getQuantity()) {
                unset($map[$plan->getPlanId()]);
            } else {
                return false;
            }
        }

        return count($map) === 0;
    }
}
