<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

namespace Rebilly\Wp\Member\Admin;

use Rebilly\Wp\Member\Entity\Form as Entity;
use Rebilly\Wp\Member\Entity\User;
use Rebilly\Wp\Member\Plugin;
use WP_Post;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Form implements Metaboxable
{
    use HasMetabox;

    private $metabox_id = 'wp_rebilly_m_form_data';

    public function __construct()
    {
        $this->metabox_init([
            'id' => 'wp_rebilly_m_form_data',
            'label' => __('Rebilly', REBILLY_WP_LANG_DOMAIN),
            'screen' => Entity::postType(),
            'priority' => 'high',
            'classes' => ['wp_rebilly_m__metabox'],
        ]);

        add_action('admin_menu', [$this, 'registerMenu']);
        add_action('edit_form_after_title', [$this, 'afterTitle']);
        add_action('admin_action_rebilly_form_duplicate', [$this, 'duplicate']);
        add_filter('post_row_actions', [$this, 'rowActions'], 10, 2);
    }

    /**
     * @param WP_Post $post
     */
    public function afterTitle($post): void
    {
        if ($post->post_type === Entity::postType()) {
            renderIsolate(REBILLY_WP_DIR . 'template/admin/shortcode-info.php', compact('post'));
        }
    }

    /**
     * @param array $actions
     * @param WP_Post $post
     * @return array
     */
    public function rowActions(array $actions, WP_Post $post): array
    {
        if (current_user_can('edit_posts') && $post->post_type === Entity::postType()) {
            $url = wp_nonce_url('admin.php?action=rebilly_form_duplicate&post=' . $post->ID, 'rebilly_form_duplicate', 'nonce');
            /** @noinspection HtmlUnknownTarget */
            $actions['duplicate'] = sprintf(
                '<a href="%s" title="%s">%s</a>',
                $url,
                __('Duplicate this form', REBILLY_WP_LANG_DOMAIN),
                __('Duplicate', REBILLY_WP_LANG_DOMAIN)
            );
        }

        return $actions;
    }

    public function registerMenu(): void
    {
        add_submenu_page(
            'rebilly_settings',
            __('Rebilly Forms', REBILLY_WP_LANG_DOMAIN),
            __('Forms', REBILLY_WP_LANG_DOMAIN),
            'manage_options',
            'edit.php?post_type=' . Entity::postType()
        );
    }

    public function duplicate(): void
    {
        if (($_REQUEST['action'] ?? '') !== 'rebilly_form_duplicate' || !is_numeric($_REQUEST['post'] ?? '')) {
            wp_die('No post to duplicate has been supplied!');
        }

        if (!wp_verify_nonce($_REQUEST['nonce'] ?? '', 'rebilly_form_duplicate') || !current_user_can('edit_posts')) {
            return;
        }

        $postId = absint($_REQUEST['post'] ?? '');
        $user = User::find();

        if ($form = Entity::find($postId)) {
            $copyLabel = __('Copy', REBILLY_WP_LANG_DOMAIN);
            $newPostId = wp_insert_post([
                'comment_status' => $form->comment_status,
                'ping_status' => $form->ping_status,
                'post_author' => $user ? $user->ID : null,
                'post_content' => $form->post_content,
                'post_excerpt' => $form->post_excerpt,
                'post_name' => $form->post_name,
                'post_parent' => $form->post_parent,
                'post_password' => $form->post_password,
                'post_status' => 'publish',
                'post_title' => "{$form->post_title} ($copyLabel)",
                'post_type' => $form->post_type,
                'to_ping' => $form->to_ping,
                'menu_order' => $form->menu_order,
            ]);
            $metaKeys = implode(',', array_map(function ($val) {
                return "'{$val}'";
            }, Entity::$meta));

            global $wpdb;
            $wpdb->query("INSERT INTO {$wpdb->postmeta}(post_id, meta_key, meta_value) SELECT {$newPostId} as post_id, meta_key, meta_value FROM {$wpdb->postmeta} WHERE post_id = {$postId} AND meta_key IN ({$metaKeys})");

            wp_redirect(admin_url("post.php?action=edit&post=$newPostId"));
            exit;
        }
        wp_die('Form creation failed, could not find original form: ' . $postId);
    }

    public function metabox_fields($metabox_id, $post): array
    {
        $plans = Plugin::api()->plan()->allNames();
        $pages = [
            '' => 'None',
        ];
        foreach (get_posts(['post_type' => 'page', 'posts_per_page' => -1]) as $page) {
            $pages[$page->ID] = get_the_title($page);
        }

        return [
            [
                'label' => 'Role',
                'name' => 'role',
                'type' => 'select',
                'options' => Entity::roles(),
            ],
            [
                'label' => 'Plans',
                'name' => 'plan',
                'type' => 'plans',
                'options' => $plans,
                'width' => 50,
                'condition' => [
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Bump',
                'description' => __('If selected in form, this will replace plans defined previously. Addons will not be removed. Does nothing if left empty.', REBILLY_WP_LANG_DOMAIN),
                'name' => 'bump',
                'type' => 'plans',
                'options' => $plans,
                'width' => 50,
                'condition' => [
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Addons',
                'name' => 'addons',
                'type' => 'addons',
                'options' => $plans,
                'width' => 50,
                'condition' => [
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Allow guest subscription',
                'label_checked' => '<i class="dashicons dashicons-yes color-green"></i> ' . __('Yes', REBILLY_WP_LANG_DOMAIN),
                'label_unchecked' => '<i class="dashicons dashicons-no color-red"></i> ' . __('No', REBILLY_WP_LANG_DOMAIN),
                'name' => 'allow-guest',
                'type' => 'checkbox-inline',
                'esc_labels' => false,
                'width' => 50,
            ],
            [
                'label' => 'Enable signup',
                'name' => 'enable-signup',
                'label_checked' => '<i class="dashicons dashicons-yes color-green"></i> ' . __('Yes', REBILLY_WP_LANG_DOMAIN),
                'label_unchecked' => '<i class="dashicons dashicons-no color-red"></i> ' . __('No', REBILLY_WP_LANG_DOMAIN),
                'description' => __('If enabled will auto-create user in wordpress. Your form must also contain such field names: first_name, last_name and email.', REBILLY_WP_LANG_DOMAIN),
                'type' => 'checkbox-inline',
                'esc_labels' => false,
                'width' => 50,
                'condition' => [
                    'allow-guest' => true,
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Confirmation email strategy',
                'name' => 'confirmation-email',
                'label_checked' => '<i class="dashicons dashicons-yes color-green"></i> ' . __('Rebilly', REBILLY_WP_LANG_DOMAIN),
                'label_unchecked' => '<i class="dashicons dashicons-wordpress"></i> ' . __('WordPress', REBILLY_WP_LANG_DOMAIN),
                'description' => __('If Rebilly is checked – confirmation email will be sent according to Rules Engine settings.', REBILLY_WP_LANG_DOMAIN),
                'type' => 'checkbox-inline',
                'esc_labels' => false,
                'width' => 50,
                'condition' => [
                    'allow-guest' => true,
                ],
            ],
            [
                'label' => 'Success page',
                'name' => 'page-success-id',
                'type' => 'select',
                'options' => $pages,
                'width' => 50,
                'condition' => [
                    'page-success-type' => true,
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Success page URL',
                'name' => 'page-success-url',
                'type' => 'text',
                'width' => 50,
                'condition' => [
                    'page-success-type' => false,
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Success page type',
                'name' => 'page-success-type',
                'label_checked' => '<i class="dashicons dashicons-admin-page"></i> ' . __('Existing page', REBILLY_WP_LANG_DOMAIN),
                'label_unchecked' => '<i class="dashicons dashicons-admin-links"></i> ' . __('Url', REBILLY_WP_LANG_DOMAIN),
                'type' => 'checkbox-inline',
                'esc_labels' => false,
                'width' => 50,
                'condition' => [
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Pending page',
                'name' => 'page-pending-id',
                'type' => 'select',
                'options' => $pages,
                'width' => 50,
                'condition' => [
                    'page-pending-type' => true,
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Pending page URL',
                'name' => 'page-pending-url',
                'type' => 'text',
                'width' => 50,
                'condition' => [
                    'page-pending-type' => false,
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Pending page type',
                'name' => 'page-pending-type',
                'label_checked' => '<i class="dashicons dashicons-admin-page"></i> ' . __('Existing page', REBILLY_WP_LANG_DOMAIN),
                'label_unchecked' => '<i class="dashicons dashicons-admin-links"></i> ' . __('Url', REBILLY_WP_LANG_DOMAIN),
                'type' => 'checkbox-inline',
                'esc_labels' => false,
                'width' => 50,
                'condition' => [
                    'role' => Entity::ROLE_SUBSCRIPTION,
                ],
            ],
            [
                'label' => 'Fields',
                'name' => 'fields',
                'type' => 'flexible-fields',
            ],
        ];
    }

    public function metabox_data($metabox_id, $post): array
    {
        $data = Entity::find($post)->metaAll(true);
        $data['page-success-type'] = ($data['page-success']['type'] ?? 'post') === 'post';
        $data['page-success-id'] = $data['page-success']['id'] ?? 0;
        $data['page-success-url'] = $data['page-success']['url'] ?? '';
        $data['page-pending-type'] = ($data['page-pending']['type'] ?? 'post') === 'post';
        $data['page-pending-id'] = $data['page-pending']['id'] ?? 0;
        $data['page-pending-url'] = $data['page-pending']['url'] ?? '';
        unset($data['page-success'], $data['page-pending']);

        $data['plan'] = $this->readPlans($data['plan'] ?? []);
        $data['bump'] = $this->readPlans($data['bump'] ?? []);

        return $data;
    }

    public function metabox_save_fields($metabox_id, $post, $fields): void
    {
        $fields['page-success'] = [
            'type' => $fields['page-success-type'] ? 'post' : 'url',
            'id' => $fields['page-success-id'],
            'url' => $fields['page-success-url'],
        ];
        $fields['page-pending'] = [
            'type' => $fields['page-pending-type'] ? 'post' : 'url',
            'id' => $fields['page-pending-id'],
            'url' => $fields['page-pending-url'],
        ];
        unset($fields['page-success-type'], $fields['page-success-id'], $fields['page-success-url'],
            $fields['page-pending-type'], $fields['page-pending-id'], $fields['page-pending-url']);

        $fields['plan'] = $this->updatePlans($fields['plan'] ?? []);
        $fields['bump'] = $this->updatePlans($fields['bump'] ?? []);

        Entity::find($post)->metaUpdate($fields, true);
    }

    private function readPlans($plans): array
    {
        $plans = is_array($plans) ? $plans : (array) ($plans ?? []);

        return array_map(function ($plan) {
            return is_string($plan) ? [
                'plan' => $plan,
                'mode' => false,
                'quantity' => 1,
            ] : $plan;
        }, $plans);
    }

    private function updatePlans($plans): array
    {
        $plans = is_array($plans) ? $plans : (array) ($plans ?? []);
        $existingPlans = array_filter(array_values($plans), function ($plan) {
            return !blank($plan['plan']);
        });

        return array_map(function ($plan) {
            $plan['mode'] = $plan['mode'] ?? false;
            $plan['quantity'] = min(100, max(1, (int) ($plan['quantity'] ?? 1)));

            return $plan;
        }, $existingPlans);
    }
}
