<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

namespace Rebilly\Wp\Member\Admin;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

trait HasMetabox
{
    private $metaboxes = [];

    public function metabox_init()
    {
        if (!is_admin()) {
            return;
        }

        $this->metabox_populate(func_get_args());

        add_action('load-post.php', [$this, 'metabox_hooks']);
        add_action('load-post-new.php', [$this, 'metabox_hooks']);
    }

    private function metabox_populate($metaboxes)
    {
        foreach ($metaboxes as $metabox) {
            $this->metaboxes[$metabox['id']] = $metabox;
        }

        return $this->metaboxes;
    }

    public function metabox_hooks()
    {
        add_action('add_meta_boxes', [$this, 'metabox_register'], 0);
        add_action('save_post', [$this, 'metabox_save'], 10, 2);
    }

    public function metabox_register()
    {
        foreach ($this->metaboxes as $metabox) {
            add_meta_box(
                $metabox['id'],
                __($metabox['label'], REBILLY_WP_LANG_DOMAIN),
                [$this, "metabox_render_{$metabox['id']}"],
                $metabox['screen'],
                $metabox['context'] ?? 'normal',
                $metabox['priority'] ?? 'default'
            );
            $screens = is_array($metabox['screen']) ? $metabox['screen'] : [$metabox['screen']];
            foreach ($screens as $screen) {
                add_filter("postbox_classes_{$screen}_{$metabox['id']}", [$this, "metabox_classes_{$metabox['id']}"]);
            }
        }
    }

    public function metabox_classes($metabox_id, $classes)
    {
        return array_merge($classes, $this->metaboxes[$metabox_id]['classes'] ?? []);
    }

    public function metabox_render($metabox_id, $post)
    {
        if (!array_key_exists($metabox_id, $this->metaboxes)) {
            return;
        }
        $metabox = $this->metaboxes[$metabox_id];
        wp_nonce_field("{$metabox['id']}_action", "{$metabox['id']}_nonce");

        wp_rebilly_m_fields($this->metabox_fields($metabox_id, $post), $this->metabox_data($metabox_id, $post));
    }

    public function metabox_save($post_id, $post)
    {
        foreach ($this->metaboxes as $metabox) {
            $nonce_name = $_POST["{$metabox['id']}_nonce"] ?? '';
            $nonce_action = "{$metabox['id']}_action";

            if (!isset($nonce_name) ||
                !wp_verify_nonce($nonce_name, $nonce_action) ||
                !current_user_can('edit_post', $post_id) ||
                wp_is_post_autosave($post_id) ||
                wp_is_post_revision($post_id)) {
                continue;
            }

            $data = $_POST;

            // Checkboxes should be saved as unchecked when they are not present in $_POST
            foreach ($this->metabox_fields($metabox['id'], null) as $field) {
                if (in_array($field['type'] ?? 'text', ['checkbox', 'checkbox-inline'], true) &&
                    !isset($data[$field['name']])) {
                    $data[$field['name']] = 0;
                }
            }

            $this->metabox_save_fields($metabox['id'], $post, $data);
        }
    }

    public function __call($name, $arguments)
    {
        $prefixes = [
            'metabox_classes_' => 'metabox_classes',
            'metabox_render_' => 'metabox_render',
        ];
        $name_lower = strtolower($name);
        foreach ($prefixes as $prefix => $target) {
            if (substr($name_lower, 0, strlen($prefix)) == $prefix) {
                array_unshift($arguments, substr($name, strlen($prefix)));

                return call_user_func_array([$this, $target], $arguments);
            }
        }

        return null;
    }
}
