<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

namespace Rebilly\Wp\Member\Admin\Page;

use Rebilly\Wp\Member\Plugin;
use Rebilly\Wp\Member\Settings;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class SettingsPage
{
    /** @var static */
    protected static $_instance = false;

    private function __construct()
    {
    }

    /**
     * @return static
     */
    public static function instance()
    {
        if (static::$_instance === false) {
            static::$_instance = new static();
        }

        return static::$_instance;
    }

    public static function register(): void
    {
        add_action('admin_init', [static::instance(), 'registerOptions']);
        add_action('admin_menu', [static::instance(), 'registerMenu']);
    }

    public function registerMenu(): void
    {
        add_menu_page(
            __('Rebilly', REBILLY_WP_LANG_DOMAIN),
            __('Rebilly', REBILLY_WP_LANG_DOMAIN),
            'administrator',
            'rebilly_settings',
            [$this, 'page_settings'],
            REBILLY_WP_URL . 'images/logo.svg',
            30
        );
        add_submenu_page(
            'rebilly_settings',
            __('Settings', REBILLY_WP_LANG_DOMAIN),
            __('Settings', REBILLY_WP_LANG_DOMAIN),
            'manage_options',
            'rebilly_settings',
            [$this, 'page_settings']
        );
    }

    public function registerOptions(): void
    {
        $sections = [
            'wp_rebilly_m_settings_rebilly' => [
                'name' => 'API',
                'fields' => [
                    'publishable_key' => 'Publishable Key',
                    'secret_key' => 'Secret Key',
                    'website_id' => 'Website',
                    'is_sandbox' => 'Environment',
                ],
            ],
            'wp_rebilly_m_settings_customers' => [
                'name' => 'Customers',
                'fields' => [
                    'auto_register' => 'Auto sign-up new customers',
                    'send_confirmation' => 'Send Email',
                ],
            ],
            'wp_rebilly_m_settings_membership' => [
                'name' => 'Membership',
                'fields' => [
                    'post_types' => 'Enabled post types',
                    'override_roles' => 'Restriction bypass roles',
                    'login_redirect' => 'Redirect after log in to',
                    'restrict_non_admins' => 'Regular users can access dashboard and profile',
                ],
            ],
            'wp_rebilly_m_settings_messages' => [
                'name' => 'Messages',
                'fields' => [
                    'message_restricted' => 'On-page text displayed for non-subscribers',
                ],
            ],
        ];
        register_setting('wp_rebilly_m_settings', 'wp_rebilly_m_settings');
        foreach ($sections as $section_id => $section) {
            add_settings_section(
                $section_id,
                $section['name'],
                null,
                'wp_rebilly_m_settings'
            );
            foreach ($section['fields'] as $field => $name) {
                add_settings_field(
                    $field,
                    $name,
                    [$this, "field_$field"],
                    'wp_rebilly_m_settings',
                    $section_id
                );
            }
        }
    }

    public function page_settings(): void
    {
        include REBILLY_WP_DIR . 'template/admin/settings.php';
    }

    public function field_publishable_key(): void
    {
        wp_rebilly_m_field([
            'name' => 'wp_rebilly_m_settings[publishable_key]',
            'type' => 'text',
            'value' => $this->option('publishable_key', ''),
            'class' => 'wp_rebilly_m__secret',
        ]);
    }

    public function field_secret_key(): void
    {
        wp_rebilly_m_field([
            'name' => 'wp_rebilly_m_settings[secret_key]',
            'type' => 'text',
            'value' => $this->option('secret_key', ''),
            'class' => 'wp_rebilly_m__secret',
        ]);
    }

    public function field_website_id(): void
    {
        $websites = Plugin::api()
            ->website()
            ->allNames();

        wp_rebilly_m_field([
            'name' => 'wp_rebilly_m_settings[website_id]',
            'type' => 'select',
            'options' => $websites,
            'value' => $this->option('website_id', ''),
        ]);
    }

    public function field_is_sandbox(): void
    {
        wp_rebilly_m_field([
            'label_checked' => __('<i class="dashicons dashicons-admin-appearance color-red"></i> Sandbox', REBILLY_WP_LANG_DOMAIN),
            'label_unchecked' => __('<i class="dashicons dashicons-admin-plugins color-green"></i> Live', REBILLY_WP_LANG_DOMAIN),
            'name' => 'wp_rebilly_m_settings[is_sandbox]',
            'type' => 'checkbox-inline',
            'esc_labels' => false,
            'value' => $this->option('is_sandbox', false),
        ]);
    }

    public function field_auto_register(): void
    {
        wp_rebilly_m_field([
            'label_checked' => __('<i class="dashicons dashicons-yes color-green"></i> Yes', REBILLY_WP_LANG_DOMAIN),
            'label_unchecked' => __('<i class="dashicons dashicons-no color-red"></i> No', REBILLY_WP_LANG_DOMAIN),
            'name' => 'wp_rebilly_m_settings[auto_register]',
            'description' => __('Should user be registered when customer created in Rebilly', REBILLY_WP_LANG_DOMAIN),
            'type' => 'checkbox-inline',
            'esc_labels' => false,
            'value' => $this->option('auto_register', false),
        ]);
    }

    public function field_send_confirmation(): void
    {
        wp_rebilly_m_field([
            'label_checked' => __('<i class="dashicons dashicons-yes color-green"></i> Yes', REBILLY_WP_LANG_DOMAIN),
            'label_unchecked' => __('<i class="dashicons dashicons-no color-red"></i> No', REBILLY_WP_LANG_DOMAIN),
            'name' => 'wp_rebilly_m_settings[auto_register_send_confirmation]',
            'type' => 'checkbox-inline',
            'esc_labels' => false,
            'value' => $this->option('auto_register_send_confirmation', false),
        ]);
    }

    public function field_post_types(): void
    {
        $post_types = [];
        foreach (get_post_types(['public' => true], 'object') as $key => $post_type) {
            $post_types[$key] = $post_type->label;
        }

        wp_rebilly_m_field([
            'name' => 'wp_rebilly_m_settings[post_types]',
            'type' => 'select',
            'options' => $post_types,
            'multiple' => true,
            'value' => $this->option('post_types', ['post', 'page']),
        ]);
    }

    public function field_override_roles(): void
    {
        global $wp_roles;

        $roles = [];
        foreach ($wp_roles->roles as $key => $role) {
            $roles[$key] = $role['name'];
        }

        wp_rebilly_m_field([
            'name' => 'wp_rebilly_m_settings[override_roles]',
            'type' => 'select',
            'options' => $roles,
            'multiple' => true,
            'value' => $this->option('override_roles', ['administrator', 'editor', 'author']),
        ]);
    }

    public function field_login_redirect(): void
    {
        $pages = [
            '' => 'Default',
        ];
        foreach (get_posts(['post_type' => 'page', 'posts_per_page' => -1]) as $page) {
            $pages[$page->ID] = get_the_title($page);
        }

        wp_rebilly_m_field([
            'name' => 'wp_rebilly_m_settings[login_redirect]',
            'type' => 'select',
            'options' => $pages,
            'value' => $this->option('login_redirect', ''),
        ]);
    }

    public function field_restrict_non_admins(): void
    {
        wp_rebilly_m_field([
            'label_checked' => __('<i class="dashicons dashicons-no color-red"></i> No', REBILLY_WP_LANG_DOMAIN),
            'label_unchecked' => __('<i class="dashicons dashicons-yes color-green"></i> Yes', REBILLY_WP_LANG_DOMAIN),
            'name' => 'wp_rebilly_m_settings[restrict_non_admins]',
            'type' => 'checkbox-inline',
            'esc_labels' => false,
            'value' => $this->option('restrict_non_admins', false),
        ]);
    }

    public function field_message_restricted(): void
    {
        wp_rebilly_m_field([
            'name' => 'wp_rebilly_m_settings[message_restricted]',
            'type' => 'textarea',
            'value' => $this->option('message_restricted', 'This content is for member only. Make sure you sign up to have access to our exclusive content.'),
        ]);
    }

    private function option(string $key, $default = null)
    {
        return Settings::get($key, $default);
    }
}
