<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Api;

use Exception;
use Rebilly\Entities\Subscription;
use Rebilly\Entities\SubscriptionCancellation;
use Rebilly\Http\Exception\UnprocessableEntityException;
use Rebilly\Rest\Collection;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class SubscriptionService
 * @package Rebilly\Wp\Member\Api
 */
class SubscriptionService extends Service
{
    /**
     * @param string $id
     *
     * @return Subscription|null
     */
    public function get(string $id): ?Subscription
    {
        if (!blank($id)) {
            try {
                return $this->api()
                    ->subscriptions()
                    ->load($id);
            } catch (Exception $ex) {
            }
        }

        return null;
    }

    /**
     * @param string $id
     *
     * @return SubscriptionCancellation|null
     */
    public function getCancellation(string $id): ?SubscriptionCancellation
    {
        if (!blank($id)) {
            try {
                return $this->api()
                    ->subscriptionCancellations()
                    ->load($id);
            } catch (Exception $ex) {
            }
        }

        return null;
    }

    /**
     * @param array $params
     *
     * @return Subscription[]|Collection
     */
    public function find(array $params): Collection
    {
        try {
            return $this->api()
                ->subscriptions()
                ->search($params);
        } catch (Exception $ex) {
        }

        return new Collection(new Subscription());
    }

    /**
     * @param string $customerId
     * @param int $page
     * @param int $perPage
     *
     * @return Subscription[]|Collection
     */
    public function findByCustomer(string $customerId, int $page = 1, int $perPage = 10): Collection
    {
        return $this->find([
            'filter' => "customerId:$customerId;websiteId:{$this->getWebsiteId()}",
            'sort' => '-createdTime',
            'limit' => $perPage,
            'offset' => $perPage * ($page - 1),
        ]);
    }

    /**
     * @param string $customerId
     * @param int $page
     * @param int $perPage
     *
     * @return Subscription[]|Collection
     */
    public function findPendingByCustomer(string $customerId, int $page = 1, int $perPage = 10): Collection
    {
        return $this->find([
            'filter' => "customerId:$customerId;websiteId:{$this->getWebsiteId()};status:pending",
            'sort' => '-createdTime',
            'limit' => $perPage,
            'offset' => $perPage * ($page - 1),
        ]);
    }

    /**
     * @param string $customerId
     *
     * @return Subscription[]|Collection
     */
    public function findActive(string $customerId): Collection
    {
        return $this->find([
            'filter' => "customerId:{$customerId};status:active",
            'sort' => '-createdTime',
        ]);
    }

    /**
     * @param Subscription|array $data
     *
     * @throws UnprocessableEntityException
     * @return Subscription|null
     */
    public function create($data): ?Subscription
    {
        try {
            return $this->api()
                ->subscriptions()
                ->create($data);
        } catch (Exception $ex) {
            if ($ex instanceof UnprocessableEntityException) {
                throw $ex;
            }
        }

        return null;
    }

    /**
     * @param string $id
     * @param Subscription|array $data
     *
     * @throws UnprocessableEntityException
     * @return Subscription|null
     */
    public function update(string $id, $data): ?Subscription
    {
        try {
            return $this->api()
                ->subscriptions()
                ->update($id, $data);
        } catch (Exception $ex) {
            if ($ex instanceof UnprocessableEntityException) {
                throw $ex;
            }
        }

        return null;
    }

    /**
     * @param SubscriptionCancellation|array $data
     *
     * @throws UnprocessableEntityException
     * @return SubscriptionCancellation|null
     */
    public function cancel($data): ?SubscriptionCancellation
    {
        try {
            return $this->api()
                ->subscriptionCancellations()
                ->create($data);
        } catch (Exception $ex) {
            if ($ex instanceof UnprocessableEntityException) {
                throw $ex;
            }
        }

        return null;
    }
}
