<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

namespace Rebilly\Wp\Member;

use Rebilly\Http\Exception\ClientException;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class RulesEngine
{
    public const SETTINGS_KEY = 'rebilly-settings-rules_engine';

    /** @var Plugin */
    private $plugin;

    private $_settings;

    public function __construct(Plugin $plugin)
    {
        $this->plugin = $plugin;
        $this->_settings = apply_filters('rebilly_webhook_settings', get_option(self::SETTINGS_KEY, []));
        if (!is_array($this->_settings)) {
            $this->_settings = [];
            update_option('wp_rebilly_m_webhook_settings', $this->_settings);
        }

        if ($plugin->isConnected()) {
            $this->load_hooks();
        }
    }

    public function load_hooks(): void
    {
        if (!$this->isRegistered() && !$this->register()) {
            rebilly_admin_notice(
                __('Failed to setup Rebilly RulesEngine. Please, check API keys and contact support.', REBILLY_WP_LANG_DOMAIN),
                'error',
                true
            );
        }
    }

    private function settings($key = null, $default = null)
    {
        if ($key === null) {
            return $this->_settings;
        }
        if (is_array($key)) {
            $this->_settings = $default === true ? $key : array_merge($this->_settings, $key);

            return update_option(self::SETTINGS_KEY, $this->_settings);
        }

        return $this->_settings[$key] ?? $default;
    }

    private function isRegistered(): bool
    {
        $required = ['fields', 'bind'];
        foreach ($required as $key) {
            if (blank($this->settings($key))) {
                return false;
            }
        }

        return true;
    }

    private function register(): bool
    {
        if ($this->isRegistered()) {
            return true;
        }

        $websiteId = Settings::get('website_id', '');

        try {
            $fields = $this->settings('fields');
            if (blank($fields)) {
                $fields = $this->plugin->getApi()
                    ->customField()
                    ->registerRulesEngine($websiteId);
            }

            // Persist fields
            $this->settings(['fields' => $fields]);

            $bind = $this->plugin->getApi()
                ->rulesEngine()
                ->registerConfirmationEmail($websiteId, $fields);
            if ($bind === null) {
                return false;
            }

            // Persist bind
            $this->settings(['bind' => $bind]);
        } catch (ClientException $e) {
            return false;
        }

        return true;
    }

    public function getFields(): array
    {
        return $this->settings('fields', []);
    }
}
