<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Shortcode;

use Rebilly\Wp\Member\Action\Action;
use Rebilly\Wp\Member\Entity\Form as Entity;
use Rebilly\Wp\Member\Entity\User;
use Rebilly\Wp\Member\Plugin;
use Rebilly\Wp\Member\Reformer\Reformer;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Form implements ShortcodeContract
{
    /**
     * @return string
     */
    public static function getName(): string
    {
        return 'wp_rebilly_m_form';
    }

    /**
     * @param array|string $attrs
     * @return string
     */
    public function render($attrs = []): string
    {
        $shortcodeName = self::getName();
        if (blank($attrs) || !($form = Entity::find($attrs[0]))) {
            return __("Use shortcode [{$shortcodeName} {form_id}] with valid form_id", REBILLY_WP_LANG_DOMAIN);
        }
        if (!Plugin::instance()->isConnected()) {
            return __('Please fill keys at settings page to enable forms', REBILLY_WP_LANG_DOMAIN);
        }

        $actionName = self::action($form->getRole());
        $action = $actionName ? Action::get($actionName) : null;
        $user = User::find();
        $isAuthenticated = (bool) $user;

        $options = array_slice($attrs, 1);
        foreach ($options as $option) {
            [$key, $value] = explode(':', $option);
            if ($key === 'only' && (
                    ($value === 'auth' && !$isAuthenticated) ||
                    ($value === 'guest' && $isAuthenticated)
                )) {
                return '';
            }
        }

        $preFill = $this->collectQueryStringParams();
        $env = compact('form', 'action', 'user', 'isAuthenticated', 'preFill');

        $components = $form->getComponents();
        $dom = Reformer::renderComponents($components, $env);
        $styles = $this->renderCss($form->getStyles());

        $googleFonts = $this->extractGoogleFonts($form->getStyles());
        if (count($googleFonts) > 0) {
            $families = implode('|', array_map('urlencode', $googleFonts));
            $styles = "@import url('https://fonts.googleapis.com/css?family=$families');" . $styles;
        }

        return renderIsolate(
            REBILLY_WP_DIR . 'template/front/form.php',
            compact('form', 'googleFonts', 'styles', 'dom'),
            true
        );
    }

    /**
     * @param array $styles
     * @return string|null
     */
    private function renderCss(array $styles): ?string
    {
        $formSelector = '.reformer-wrap';
        $code = '';
        foreach ($styles as $style) {
            $media = $this->makeAtRule($style);
            $declaration = $this->makeDeclaration($style, $formSelector);
            if (blank($declaration)) {
                continue;
            }
            $code .= $media ? $media . '{' . $declaration . '}' : $declaration;
        }

        return $code;
    }

    private function makeDeclaration(array $style, string $formSelector): string
    {
        $result = '';
        $selector = $this->makeSelector($style, $formSelector);
        $css = $this->makeStyles($style);
        $singleAtRule = $style['singleAtRule'];

        if (($selector || $singleAtRule) && $css) {
            $result = $singleAtRule ? $css : $selector . '{' . $css . '}';
        }

        return $result;
    }

    private function makeAtRule(array $style): string
    {
        $type = $style['atRuleType'];
        $condition = $style['mediaText'];
        if ($type) {
            $typeStr = "@$type";
        } else {
            $typeStr = $condition ? '@media' : '';
        }

        return $typeStr . ($condition && $typeStr ? " $condition" : '');
    }

    private function makeSelector(array $style, string $prefix = ''): string
    {
        $prefix = trim($prefix);
        if ($prefix !== '') {
            $prefix .= ' ';
        }
        $css = $prefix;
        foreach ($style['selectors'] as $selector) {
            switch ($selector['type'] ?? 1) {
                case 1:
                    $css .= '.';

                    break;
                case 2:
                    $css .= '#';

                    break;
            }
            $css .= $selector['name'];
        }
        $state = $style['state'] ?? '';
        if (!blank($state)) {
            $css .= ':' . $state;
        }

        return $css;
    }

    private function makeStyles(array $style): string
    {
        $important = (bool) ($style['important'] ?? 0) ? ' !important' : '';
        $css = '';
        foreach ($style['style'] as $property => $value) {
            $css .= $property . ':' . $value . $important . ';';
        }

        return $css;
    }

    private function extractGoogleFonts(array $styles): array
    {
        $googleFonts = [
            'Karla',
            'Lora',
            'Frank Ruhl Libre',
            'Playfair Display',
            'Archivo',
            'Spectral',
            'Fjalla One',
            'Roboto',
            'Montserrat',
            'Rubik',
            'Source Sans',
            'Cardo',
            'Cormorant',
            'Work Sans',
            'Rakkas',
            'Concert One',
            'Yatra One',
            'Arvo',
            'Lato',
            'Abril FatFace',
            'Ubuntu',
            'PT Serif',
            'Old Standard TT',
            'Oswald',
            'Raleway',
            'Open Sans',
            'Merriweather',
            'Source Sans Pro',
        ];
        $fonts = [];
        foreach ($styles as $style) {
            foreach ($style['style'] as $property => $value) {
                if ($property === 'font-family') {
                    $font = explode(',', $value)[0];
                    if (in_array($font, $googleFonts, true)) {
                        $fonts[] = $font;
                    }
                }
            }
        }

        return array_unique($fonts);
    }

    private static function action(string $role): ?string
    {
        switch ($role) {
            case Entity::ROLE_LOGIN:
                return 'rebilly_form_login';
            case Entity::ROLE_SUBSCRIPTION:
                return 'rebilly_form_subscribe';
            case Entity::ROLE_ADD_CARD:
                return 'rebilly_card_add';
            default:
                return null;
        }
    }

    private function collectQueryStringParams(): array
    {
        $availableParams = [
            'address',
            'address2',
            'city',
            'region',
            'country',
            'postalCode',
        ];
        $params = [];
        foreach($availableParams as $key) {
            if(isset($_GET[$key])) {
                $params[$key] = sanitize_text_field($_GET[$key]);
            }
        }

        return $params;
    }
}
